unit Modules;

interface

uses
  Windows,
  SharedTypes;

type
  //  
  TModule=class
  private
    FFileName:String;           //   
    FHandle:THandle;            //  
    FModuleInfo:TModuleInfo;    //  
    //   
    FGetModuleInfo:TGetModuleInfo;      //   
    FInitialize:TInitialize;            // 
    FRun:TRun;                          // 
    FTerminate:TTerminate;              // 
    FSetData:TSetData;                  //  
  public
    constructor Create(AFileName:String;var IsValidModule:Boolean);
    destructor Destroy;override;
    //     
    function GetModuleInfo:TModuleInfo; //   
    procedure Initialize;               // 
    procedure Run;                      // 
    procedure Terminate;                // 
    procedure SetData(Kind:TDataKind;   //  
      const Buffer;Size:Integer);
    //  
    property FileName:String read FFileName;
    property Handle:THandle read FHandle;
    property ModuleInfo:TModuleInfo read FModuleInfo;
  end;

implementation

constructor TModule.Create(AFileName:String;var IsValidModule:Boolean);
begin
  //  
  IsValidModule:=False;
  // 
  FHandle:=LoadLibrary(PChar(AFileName));
  if FHandle>0 then begin
    //    
    FGetModuleInfo:=GetProcAddress(FHandle,ProcedureNames[piGetModuleInfo]);
    FInitialize:=GetProcAddress(FHandle,ProcedureNames[piInitialize]);
    FRun:=GetProcAddress(FHandle,ProcedureNames[piRun]);
    FTerminate:=GetProcAddress(FHandle,ProcedureNames[piTerminate]);
    FSetData:=GetProcAddress(FHandle,ProcedureNames[piSetData]);
    //  
    FModuleInfo:=GetModuleInfo;
    Initialize;
    IsValidModule:=True;
  end;
end;

destructor TModule.Destroy;
begin
  FreeLibrary(FHandle);
  inherited Destroy;
end;

function TModule.GetModuleInfo:TModuleInfo;
begin
  if @FGetModuleInfo<>nil then
    Result:=FGetModuleInfo;
end;

procedure TModule.Initialize;
begin
  if @FInitialize<>nil then
    FInitialize;
end;

procedure TModule.Run;
begin
  if @FRun<>nil then
    FRun;
end;

procedure TModule.Terminate;
begin
  if @FTerminate<>nil then
    FTerminate;
end;

procedure TModule.SetData(Kind:TDataKind;const Buffer;Size:Integer);
begin
  if @FSetData<>nil then
    FSetData(Kind,Buffer,Size);
end;

end.

